@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2010  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,27 +18,34 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle;
 
-import java.io.File;
-
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.Locale;
-import java.util.Stack;
-import java.util.StringTokenizer;
-
+import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 import com.puppycrawl.tools.checkstyle.api.AuditEvent;
 import com.puppycrawl.tools.checkstyle.api.AuditListener;
 import com.puppycrawl.tools.checkstyle.api.AutomaticBean;
 import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
 import com.puppycrawl.tools.checkstyle.api.Configuration;
 import com.puppycrawl.tools.checkstyle.api.Context;
+import com.puppycrawl.tools.checkstyle.api.FastStack;
 import com.puppycrawl.tools.checkstyle.api.FileSetCheck;
+import com.puppycrawl.tools.checkstyle.api.FileText;
 import com.puppycrawl.tools.checkstyle.api.Filter;
 import com.puppycrawl.tools.checkstyle.api.FilterSet;
 import com.puppycrawl.tools.checkstyle.api.LocalizedMessage;
 import com.puppycrawl.tools.checkstyle.api.MessageDispatcher;
 import com.puppycrawl.tools.checkstyle.api.SeverityLevel;
+import com.puppycrawl.tools.checkstyle.api.SeverityLevelCounter;
 import com.puppycrawl.tools.checkstyle.api.Utils;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.UnsupportedEncodingException;
+import java.nio.charset.Charset;
+import java.util.List;
+import java.util.Locale;
+import java.util.Set;
+import java.util.StringTokenizer;
+import java.util.TreeSet;
 
 /**
  * This class provides the functionality to check a set of files.
@@ -46,73 +53,21 @@ import com.puppycrawl.tools.checkstyle.api.Utils;
  * @author <a href="mailto:stephane.bailliez@wanadoo.fr">Stephane Bailliez</a>
  * @author lkuehne
  */
-public class Checker extends AutomaticBean
-    implements MessageDispatcher
+public class Checker extends AutomaticBean implements MessageDispatcher
 {
-    /**
-     * An AuditListener that maintains the number of errors.
-     */
-    private class ErrorCounter implements AuditListener
-    {
-        /** keeps track of the number of errors */
-        private int mCount;
-
-        /** @see AuditListener */
-        public void addError(AuditEvent aEvt)
-        {
-            if (SeverityLevel.ERROR.equals(aEvt.getSeverityLevel())) {
-                mCount++;
-            }
-        }
-
-        /** @see AuditListener */
-        public void addException(AuditEvent aEvt, Throwable aThrowable)
-        {
-            mCount++;
-        }
-
-        /** @see AuditListener */
-        public void auditStarted(AuditEvent aEvt)
-        {
-            mCount = 0;
-        }
-
-        /** @see AuditListener */
-        public void fileStarted(AuditEvent aEvt)
-        {
-        }
-
-        /** @see AuditListener */
-        public void auditFinished(AuditEvent aEvt)
-        {
-        }
-
-        /** @see AuditListener */
-        public void fileFinished(AuditEvent aEvt)
-        {
-        }
-
-        /**
-         * @return the number of errors since audit started.
-         */
-        private int getCount()
-        {
-            return mCount;
-        }
-    }
-
     /** maintains error count */
-    private final ErrorCounter mCounter = new ErrorCounter();
+    private final SeverityLevelCounter mCounter = new SeverityLevelCounter(
+            SeverityLevel.ERROR);
 
     /** vector of listeners */
-    private final ArrayList mListeners = new ArrayList();
+    private final List<AuditListener> mListeners = Lists.newArrayList();
 
     /** vector of fileset checks */
-    private final ArrayList mFileSetChecks = new ArrayList();
+    private final List<FileSetCheck> mFileSetChecks = Lists.newArrayList();
 
     /** class loader to resolve classes with. **/
-    private ClassLoader mLoader =
-            Thread.currentThread().getContextClassLoader();
+    private ClassLoader mLoader = Thread.currentThread()
+            .getContextClassLoader();
 
     /** the basedir to strip off in filenames */
     private String mBasedir;
@@ -125,6 +80,9 @@ public class Checker extends AutomaticBean
     /** The factory for instantiating submodules */
     private ModuleFactory mModuleFactory;
 
+    /** The classloader used for loading Checkstyle module classes. */
+    private ClassLoader mModuleClassLoader;
+
     /** the context of all child components */
     private Context mChildContext;
 
@@ -143,31 +101,42 @@ public class Checker extends AutomaticBean
      */
     private SeverityLevel mSeverityLevel = SeverityLevel.ERROR;
 
+    /** Name of a charset */
+    private String mCharset = System.getProperty("file.encoding", "UTF-8");
+
     /**
      * Creates a new <code>Checker</code> instance.
      * The instance needs to be contextualized and configured.
      *
      * @throws CheckstyleException if an error occurs
      */
-    public Checker()
-        throws CheckstyleException
+    public Checker() throws CheckstyleException
     {
         addListener(mCounter);
     }
 
-    /** @see AutomaticBean */
-    public void finishLocalSetup()
-        throws CheckstyleException
+    @Override
+    public void finishLocalSetup() throws CheckstyleException
     {
         final Locale locale = new Locale(mLocaleLanguage, mLocaleCountry);
         LocalizedMessage.setLocale(locale);
 
         if (mModuleFactory == null) {
-            mModuleFactory = PackageNamesLoader.loadModuleFactory(
-                    this.getClass().getClassLoader());
+
+            if (mModuleClassLoader == null) {
+                throw new CheckstyleException(
+                        "if no custom moduleFactory is set, "
+                                + "moduleClassLoader must be specified");
+            }
+
+            final Set<String> packageNames = PackageNamesLoader
+                    .getPackageNames(mModuleClassLoader);
+            mModuleFactory = new PackageObjectFactory(packageNames,
+                    mModuleClassLoader);
         }
 
         final DefaultContext context = new DefaultContext();
+        context.add("charset", mCharset);
         context.add("classLoader", mLoader);
         context.add("moduleFactory", mModuleFactory);
         context.add("severity", mSeverityLevel.getName());
@@ -175,12 +144,7 @@ public class Checker extends AutomaticBean
         mChildContext = context;
     }
 
-    /**
-     * Instantiates, configures and registers a child AbstractFilter
-     * or FileSetCheck
-     * that is specified in the provided configuration.
-     * @see com.puppycrawl.tools.checkstyle.api.AutomaticBean
-     */
+    @Override
     protected void setupChild(Configuration aChildConf)
         throws CheckstyleException
     {
@@ -206,14 +170,13 @@ public class Checker extends AutomaticBean
             }
             else {
                 throw new CheckstyleException(name
-                    + " is not allowed as a child in Checker");
+                        + " is not allowed as a child in Checker");
             }
         }
-        catch (Exception ex) {
+        catch (final Exception ex) {
             // TODO i18n
-            throw new CheckstyleException(
-                    "cannot initialize module "
-                    + name + " - " + ex.getMessage(), ex);
+            throw new CheckstyleException("cannot initialize module " + name
+                    + " - " + ex.getMessage(), ex);
         }
     }
 
@@ -257,7 +220,7 @@ public class Checker extends AutomaticBean
      * Add the listener that will be used to receive events from the audit.
      * @param aListener the nosy thing
      */
-    public void addListener(AuditListener aListener)
+    public final void addListener(AuditListener aListener)
     {
         mListeners.add(aListener);
     }
@@ -279,20 +242,56 @@ public class Checker extends AutomaticBean
      * @return the total number of errors found
      * @see #destroy()
      */
-    public int process(File[] aFiles)
+    public int process(List<File> aFiles)
     {
+        // Prepare to start
         fireAuditStarted();
-        for (int i = 0; i < mFileSetChecks.size(); i++) {
-            FileSetCheck fileSetCheck = (FileSetCheck) mFileSetChecks.get(i);
-            fileSetCheck.process(aFiles);
-            fileSetCheck.destroy();
+        for (FileSetCheck fsc : mFileSetChecks) {
+            fsc.beginProcessing(mCharset);
         }
-        int errorCount = mCounter.getCount();
+
+        // Process each file
+        for (File f : aFiles) {
+            final String fileName = f.getAbsolutePath();
+            fireFileStarted(fileName);
+            final TreeSet<LocalizedMessage> fileMessages = Sets.newTreeSet();
+            try {
+                final FileText theText = new FileText(f.getAbsoluteFile(),
+                        mCharset);
+                for (FileSetCheck fsc : mFileSetChecks) {
+                    fileMessages.addAll(fsc.process(f, theText));
+                }
+            }
+            catch (final FileNotFoundException fnfe) {
+                Utils.getExceptionLogger().debug(
+                        "FileNotFoundException occured.", fnfe);
+                fileMessages.add(new LocalizedMessage(0,
+                        Defn.CHECKSTYLE_BUNDLE, "general.fileNotFound", null,
+                        null, this.getClass(), null));
+            }
+            catch (final IOException ioe) {
+                Utils.getExceptionLogger().debug("IOException occured.", ioe);
+                fileMessages.add(new LocalizedMessage(0,
+                        Defn.CHECKSTYLE_BUNDLE, "general.exception",
+                        new String[] {ioe.getMessage()}, null, this.getClass(),
+                        null));
+            }
+            fireErrors(fileName, fileMessages);
+            fireFileFinished(fileName);
+        }
+
+        // Finish up
+        for (FileSetCheck fsc : mFileSetChecks) {
+            // They may also log!!!
+            fsc.finishProcessing();
+            fsc.destroy();
+        }
+
+        final int errorCount = mCounter.getCount();
         fireAuditFinished();
         return errorCount;
     }
 
-
     /**
      * Create a stripped down version of a filename.
      * @param aFileName the original filename
@@ -331,33 +330,30 @@ public class Checker extends AutomaticBean
      */
     public String normalize(String aPath)
     {
-        final String osName =
-            System.getProperty("os.name").toLowerCase(Locale.US);
+        final String osName = System.getProperty("os.name").toLowerCase(
+                Locale.US);
         final boolean onNetWare = (osName.indexOf("netware") > -1);
 
-        String orig = aPath;
+        final String orig = aPath;
 
-        aPath = aPath.replace('/', File.separatorChar)
-            .replace('\\', File.separatorChar);
+        aPath = aPath.replace('/', File.separatorChar).replace('\\',
+                File.separatorChar);
 
         // make sure we are dealing with an absolute path
-        int colon = aPath.indexOf(":");
+        final int colon = aPath.indexOf(":");
 
         if (!onNetWare) {
             if (!aPath.startsWith(File.separator)
-                && !(aPath.length() >= 2
-                     && Character.isLetter(aPath.charAt(0))
-                     && colon == 1))
+                && !((aPath.length() >= 2)
+                     && Character.isLetter(aPath.charAt(0)) && (colon == 1)))
             {
-                String msg = aPath + " is not an absolute path";
+                final String msg = aPath + " is not an absolute path";
                 throw new IllegalArgumentException(msg);
             }
         }
         else {
-            if (!aPath.startsWith(File.separator)
-                && (colon == -1))
-            {
-                String msg = aPath + " is not an absolute path";
+            if (!aPath.startsWith(File.separator) && (colon == -1)) {
+                final String msg = aPath + " is not an absolute path";
                 throw new IllegalArgumentException(msg);
             }
         }
@@ -365,17 +361,15 @@ public class Checker extends AutomaticBean
         boolean dosWithDrive = false;
         String root = null;
         // Eliminate consecutive slashes after the drive spec
-        if ((!onNetWare
-             && aPath.length() >= 2
-             && Character.isLetter(aPath.charAt(0))
-             && aPath.charAt(1) == ':')
-            || (onNetWare && colon > -1))
+        if ((!onNetWare && (aPath.length() >= 2)
+             && Character.isLetter(aPath.charAt(0)) && (aPath.charAt(1) == ':'))
+            || (onNetWare && (colon > -1)))
         {
 
             dosWithDrive = true;
 
-            char[] ca = aPath.replace('/', '\\').toCharArray();
-            StringBuffer sbRoot = new StringBuffer();
+            final char[] ca = aPath.replace('/', '\\').toCharArray();
+            final StringBuffer sbRoot = new StringBuffer();
             for (int i = 0; i < colon; i++) {
                 sbRoot.append(Character.toUpperCase(ca[i]));
             }
@@ -386,10 +380,9 @@ public class Checker extends AutomaticBean
             root = sbRoot.toString();
 
             // Eliminate consecutive slashes after the drive spec
-            StringBuffer sbPath = new StringBuffer();
+            final StringBuffer sbPath = new StringBuffer();
             for (int i = colon + 1; i < ca.length; i++) {
-                if ((ca[i] != '\\')
-                    || (ca[i] == '\\' && ca[i - 1] != '\\'))
+                if ((ca[i] != '\\') || ((ca[i] == '\\') && (ca[i - 1] != '\\')))
                 {
                     sbPath.append(ca[i]);
                 }
@@ -413,39 +406,36 @@ public class Checker extends AutomaticBean
             }
         }
 
-        Stack s = new Stack();
+        final FastStack<String> s = FastStack.newInstance();
         s.push(root);
-        StringTokenizer tok = new StringTokenizer(aPath, File.separator);
+        final StringTokenizer tok = new StringTokenizer(aPath, File.separator);
         while (tok.hasMoreTokens()) {
-            String thisToken = tok.nextToken();
+            final String thisToken = tok.nextToken();
             if (".".equals(thisToken)) {
                 continue;
             }
             else if ("..".equals(thisToken)) {
                 if (s.size() < 2) {
                     throw new IllegalArgumentException("Cannot resolve path "
-                                                       + orig);
-                }
-                else {
-                    s.pop();
+                            + orig);
                 }
+                s.pop();
             }
             else { // plain component
                 s.push(thisToken);
             }
         }
 
-        StringBuffer sb = new StringBuffer();
+        final StringBuffer sb = new StringBuffer();
         for (int i = 0; i < s.size(); i++) {
             if (i > 1) {
                 // not before the filesystem root and not after it, since root
                 // already contains one
                 sb.append(File.separatorChar);
             }
-            sb.append(s.elementAt(i));
+            sb.append(s.peek(i));
         }
 
-
         aPath = sb.toString();
         if (dosWithDrive) {
             aPath = aPath.replace('/', '\\');
@@ -454,7 +444,7 @@ public class Checker extends AutomaticBean
     }
 
     /** @return the base directory property used in unit-test. */
-    String getBasedir()
+    public final String getBasedir()
     {
         return mBasedir;
     }
@@ -463,12 +453,8 @@ public class Checker extends AutomaticBean
     protected void fireAuditStarted()
     {
         final AuditEvent evt = new AuditEvent(this);
-        if (mFilters.accept(evt)) {
-            final Iterator it = mListeners.iterator();
-            while (it.hasNext()) {
-                final AuditListener listener = (AuditListener) it.next();
-                listener.auditStarted(evt);
-            }
+        for (AuditListener listener : mListeners) {
+            listener.auditStarted(evt);
         }
     }
 
@@ -476,63 +462,54 @@ public class Checker extends AutomaticBean
     protected void fireAuditFinished()
     {
         final AuditEvent evt = new AuditEvent(this);
-        if (mFilters.accept(evt)) {
-            final Iterator it = mListeners.iterator();
-            while (it.hasNext()) {
-                final AuditListener listener = (AuditListener) it.next();
-                listener.auditFinished(evt);
-            }
+        for (AuditListener listener : mListeners) {
+            listener.auditFinished(evt);
         }
     }
 
     /**
      * Notify all listeners about the beginning of a file audit.
-     * @param aFileName the file to be audited
+     *
+     * @param aFileName
+     *            the file to be audited
      */
     public void fireFileStarted(String aFileName)
     {
         final String stripped = getStrippedFileName(aFileName);
         final AuditEvent evt = new AuditEvent(this, stripped);
-        if (mFilters.accept(evt)) {
-            final Iterator it = mListeners.iterator();
-            while (it.hasNext()) {
-                final AuditListener listener = (AuditListener) it.next();
-                listener.fileStarted(evt);
-            }
+        for (AuditListener listener : mListeners) {
+            listener.fileStarted(evt);
         }
     }
 
     /**
      * Notify all listeners about the end of a file audit.
-     * @param aFileName the audited file
+     *
+     * @param aFileName
+     *            the audited file
      */
     public void fireFileFinished(String aFileName)
     {
         final String stripped = getStrippedFileName(aFileName);
         final AuditEvent evt = new AuditEvent(this, stripped);
-        if (mFilters.accept(evt)) {
-            final Iterator it = mListeners.iterator();
-            while (it.hasNext()) {
-                final AuditListener listener = (AuditListener) it.next();
-                listener.fileFinished(evt);
-            }
+        for (AuditListener listener : mListeners) {
+            listener.fileFinished(evt);
         }
     }
 
     /**
      * notify all listeners about the errors in a file.
+     *
      * @param aFileName the audited file
      * @param aErrors the audit errors from the file
      */
-    public void fireErrors(String aFileName, LocalizedMessage[] aErrors)
+    public void fireErrors(String aFileName, TreeSet<LocalizedMessage> aErrors)
     {
         final String stripped = getStrippedFileName(aFileName);
-        for (int i = 0; i < aErrors.length; i++) {
-            final AuditEvent evt = new AuditEvent(this, stripped, aErrors[i]);
+        for (LocalizedMessage element : aErrors) {
+            final AuditEvent evt = new AuditEvent(this, stripped, element);
             if (mFilters.accept(evt)) {
-                final Iterator it = mListeners.iterator();
-                while (it.hasNext()) {
-                    final AuditListener listener = (AuditListener) it.next();
+                for (AuditListener listener : mListeners) {
                     listener.addError(evt);
                 }
             }
@@ -584,4 +561,31 @@ public class Checker extends AutomaticBean
     {
         mLoader = aLoader;
     }
+
+    /**
+     * Sets the classloader used to load Checkstyle core and custom module
+     * classes when the module tree is being built up.
+     * If no custom ModuleFactory is being set for the Checker module then
+     * this module classloader must be specified.
+     * @param aModuleClassLoader the classloader used to load module classes
+     */
+    public final void setModuleClassLoader(ClassLoader aModuleClassLoader)
+    {
+        mModuleClassLoader = aModuleClassLoader;
+    }
+
+    /**
+     * Sets a named charset.
+     * @param aCharset the name of a charset
+     * @throws UnsupportedEncodingException if aCharset is unsupported.
+     */
+    public void setCharset(String aCharset)
+        throws UnsupportedEncodingException
+    {
+        if (!Charset.isSupported(aCharset)) {
+            final String message = "unsupported charset: '" + aCharset + "'";
+            throw new UnsupportedEncodingException(message);
+        }
+        mCharset = aCharset;
+    }
 }
