@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2010  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,20 +18,13 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle;
 
-import java.io.File;
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.Reader;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.Map;
-import java.util.Set;
-
 import antlr.RecognitionException;
 import antlr.TokenStreamException;
+import antlr.TokenStreamRecognitionException;
+
+import com.google.common.collect.HashMultimap;
+import com.google.common.collect.Multimap;
+import com.google.common.collect.Sets;
 import com.puppycrawl.tools.checkstyle.api.AbstractFileSetCheck;
 import com.puppycrawl.tools.checkstyle.api.Check;
 import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
@@ -39,13 +32,21 @@ import com.puppycrawl.tools.checkstyle.api.Configuration;
 import com.puppycrawl.tools.checkstyle.api.Context;
 import com.puppycrawl.tools.checkstyle.api.DetailAST;
 import com.puppycrawl.tools.checkstyle.api.FileContents;
+import com.puppycrawl.tools.checkstyle.api.FileText;
 import com.puppycrawl.tools.checkstyle.api.LocalizedMessage;
 import com.puppycrawl.tools.checkstyle.api.TokenTypes;
 import com.puppycrawl.tools.checkstyle.api.Utils;
-import com.puppycrawl.tools.checkstyle.grammars.GeneratedJavaRecognizer;
-import com.puppycrawl.tools.checkstyle.grammars.GeneratedJava14Recognizer;
 import com.puppycrawl.tools.checkstyle.grammars.GeneratedJavaLexer;
-import com.puppycrawl.tools.checkstyle.grammars.GeneratedJava14Lexer;
+import com.puppycrawl.tools.checkstyle.grammars.GeneratedJavaRecognizer;
+import java.io.File;
+import java.io.Reader;
+import java.io.StringReader;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.List;
+import java.util.Set;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 
 /**
  * Responsible for walking an abstract syntax tree and notifying interested
@@ -57,59 +58,14 @@ import com.puppycrawl.tools.checkstyle.grammars.GeneratedJava14Lexer;
 public final class TreeWalker
     extends AbstractFileSetCheck
 {
-    /**
-     * Overrides ANTLR error reporting so we completely control
-     * checkstyle's output during parsing. This is important because
-     * we try parsing with several grammers (with/without support for
-     * <code>assert</code>). We must not write any error messages when
-     * parsing fails because with the next grammar it might succeed
-     * and the user will be confused.
-     */
-    private static final class SilentJava14Recognizer
-        extends GeneratedJava14Recognizer
-    {
-        /**
-         * Creates a new <code>SilentJava14Recognizer</code> instance.
-         *
-         * @param aLexer the tokenstream the recognizer operates on.
-         */
-        private SilentJava14Recognizer(GeneratedJava14Lexer aLexer)
-        {
-            super(aLexer);
-        }
-
-        /**
-         * Parser error-reporting function, does nothing.
-         * @param aRex the exception to be reported
-         */
-        public void reportError(RecognitionException aRex)
-        {
-        }
-
-        /**
-         * Parser error-reporting function, does nothing.
-         * @param aMsg the error message
-         */
-        public void reportError(String aMsg)
-        {
-        }
-
-        /**
-         * Parser warning-reporting function, does nothing.
-         * @param aMsg the error message
-         */
-        public void reportWarning(String aMsg)
-        {
-        }
-    }
-
     /** default distance between tab stops */
     private static final int DEFAULT_TAB_WIDTH = 8;
 
     /** maps from token name to checks */
-    private final Map mTokenToChecks = new HashMap();
+    private final Multimap<String, Check> mTokenToChecks =
+        HashMultimap.create();
     /** all the registered checks */
-    private final Set mAllChecks = new HashSet();
+    private final Set<Check> mAllChecks = Sets.newHashSet();
     /** the distance between tab stops */
     private int mTabWidth = DEFAULT_TAB_WIDTH;
     /** cache file **/
@@ -124,12 +80,33 @@ public final class TreeWalker
     /** a factory for creating submodules (i.e. the Checks) */
     private ModuleFactory mModuleFactory;
 
+    /** controls whether we should use recursive or iterative
+     * algorithm for tree processing.
+     */
+    private final boolean mRecursive;
+
+    /** logger for debug purpose */
+    private static final Log LOG =
+        LogFactory.getLog("com.puppycrawl.tools.checkstyle.TreeWalker");
+
     /**
      * Creates a new <code>TreeWalker</code> instance.
      */
     public TreeWalker()
     {
         setFileExtensions(new String[]{"java"});
+        // Tree walker can use two possible algorithms for
+        // tree processing (iterative and recursive.
+        // Recursive is default for now.
+        final String recursive =
+            System.getProperty("checkstyle.use.recursive.algorithm", "false");
+        mRecursive = "true".equals(recursive);
+        if (mRecursive) {
+            LOG.debug("TreeWalker uses recursive algorithm");
+        }
+        else {
+            LOG.debug("TreeWalker uses iterative algorithm");
+        }
     }
 
     /** @param aTabWidth the distance between tab stops */
@@ -160,10 +137,10 @@ public final class TreeWalker
         mModuleFactory = aModuleFactory;
     }
 
-    /** @see com.puppycrawl.tools.checkstyle.api.Configurable */
+    @Override
     public void finishLocalSetup()
     {
-        DefaultContext checkContext = new DefaultContext();
+        final DefaultContext checkContext = new DefaultContext();
         checkContext.add("classLoader", mClassLoader);
         checkContext.add("messages", getMessageCollector());
         checkContext.add("severity", getSeverity());
@@ -175,11 +152,7 @@ public final class TreeWalker
         mChildContext = checkContext;
     }
 
-    /**
-     * Instantiates, configures and registers a Check that is specified
-     * in the provided configuration.
-     * @see com.puppycrawl.tools.checkstyle.api.AutomaticBean
-     */
+    @Override
     public void setupChild(Configuration aChildConf)
         throws CheckstyleException
     {
@@ -198,11 +171,8 @@ public final class TreeWalker
         registerCheck(c);
     }
 
-    /**
-     * Processes a specified file and reports all errors found.
-     * @param aFile the file to process
-     **/
-    private void process(File aFile)
+    @Override
+    protected void processFiltered(File aFile, List<String> aLines)
     {
         // check if already checked and passed the file
         final String fileName = aFile.getPath();
@@ -212,45 +182,52 @@ public final class TreeWalker
         }
 
         try {
-            getMessageDispatcher().fireFileStarted(fileName);
-            final String[] lines = Utils.getLines(fileName);
-            final FileContents contents = new FileContents(fileName, lines);
+            final FileText text = FileText.fromLines(aFile, aLines);
+            final FileContents contents = new FileContents(text);
             final DetailAST rootAST = TreeWalker.parse(contents);
             walk(rootAST, contents);
         }
-        catch (FileNotFoundException fnfe) {
-            Utils.getExceptionLogger()
-                .debug("FileNotFoundException occured.", fnfe);
-            getMessageCollector().add(
-                new LocalizedMessage(
-                    0,
-                    Defn.CHECKSTYLE_BUNDLE,
-                    "general.fileNotFound",
-                    null,
-                    this.getClass()));
-        }
-        catch (IOException ioe) {
-            Utils.getExceptionLogger().debug("IOException occured.", ioe);
-            getMessageCollector().add(
-                new LocalizedMessage(
-                    0,
-                    Defn.CHECKSTYLE_BUNDLE,
-                    "general.exception",
-                    new String[] {ioe.getMessage()},
-                    this.getClass()));
-        }
-        catch (RecognitionException re) {
+        catch (final RecognitionException re) {
             Utils.getExceptionLogger()
                 .debug("RecognitionException occured.", re);
             getMessageCollector().add(
                 new LocalizedMessage(
-                    0,
+                    re.getLine(),
+                    re.getColumn(),
                     Defn.CHECKSTYLE_BUNDLE,
                     "general.exception",
                     new String[] {re.getMessage()},
-                    this.getClass()));
+                    getId(),
+                    this.getClass(), null));
         }
-        catch (TokenStreamException te) {
+        catch (final TokenStreamRecognitionException tre) {
+            Utils.getExceptionLogger()
+                .debug("TokenStreamRecognitionException occured.", tre);
+            final RecognitionException re = tre.recog;
+            if (re != null) {
+                getMessageCollector().add(
+                    new LocalizedMessage(
+                        re.getLine(),
+                        re.getColumn(),
+                        Defn.CHECKSTYLE_BUNDLE,
+                        "general.exception",
+                        new String[] {re.getMessage()},
+                        getId(),
+                        this.getClass(), null));
+            }
+            else {
+                getMessageCollector().add(
+                    new LocalizedMessage(
+                        0,
+                        Defn.CHECKSTYLE_BUNDLE,
+                        "general.exception",
+                        new String[]
+                        {"TokenStreamRecognitionException occured."},
+                        getId(),
+                        this.getClass(), null));
+            }
+        }
+        catch (final TokenStreamException te) {
             Utils.getExceptionLogger()
                 .debug("TokenStreamException occured.", te);
             getMessageCollector().add(
@@ -259,9 +236,10 @@ public final class TreeWalker
                     Defn.CHECKSTYLE_BUNDLE,
                     "general.exception",
                     new String[] {te.getMessage()},
-                    this.getClass()));
+                    getId(),
+                    this.getClass(), null));
         }
-        catch (Throwable err) {
+        catch (final Throwable err) {
             Utils.getExceptionLogger().debug("Throwable occured.", err);
             getMessageCollector().add(
                 new LocalizedMessage(
@@ -269,17 +247,13 @@ public final class TreeWalker
                     Defn.CHECKSTYLE_BUNDLE,
                     "general.exception",
                     new String[] {"" + err},
-                    this.getClass()));
+                    getId(),
+                    this.getClass(), null));
         }
 
         if (getMessageCollector().size() == 0) {
             mCache.checkedOk(fileName, timestamp);
         }
-        else {
-            fireErrors(fileName);
-        }
-
-        getMessageDispatcher().fireFileFinished(fileName);
     }
 
     /**
@@ -290,17 +264,15 @@ public final class TreeWalker
     private void registerCheck(Check aCheck)
         throws CheckstyleException
     {
-        int[] tokens = new int[] {}; //safety initialization
-        final Set checkTokens = aCheck.getTokenNames();
+        final int[] tokens;
+        final Set<String> checkTokens = aCheck.getTokenNames();
         if (!checkTokens.isEmpty()) {
             tokens = aCheck.getRequiredTokens();
 
             //register configured tokens
             final int acceptableTokens[] = aCheck.getAcceptableTokens();
             Arrays.sort(acceptableTokens);
-            final Iterator it = checkTokens.iterator();
-            while (it.hasNext()) {
-                final String token = (String) it.next();
+            for (String token : checkTokens) {
                 try {
                     final int tokenId = TokenTypes.getTokenId(token);
                     if (Arrays.binarySearch(acceptableTokens, tokenId) >= 0) {
@@ -308,7 +280,7 @@ public final class TreeWalker
                     }
                     // TODO: else log warning
                 }
-                catch (IllegalArgumentException ex) {
+                catch (final IllegalArgumentException ex) {
                     throw new CheckstyleException("illegal token \""
                         + token + "\" in check " + aCheck, ex);
                 }
@@ -317,8 +289,8 @@ public final class TreeWalker
         else {
             tokens = aCheck.getDefaultTokens();
         }
-        for (int i = 0; i < tokens.length; i++) {
-            registerCheck(tokens[i], aCheck);
+        for (int element : tokens) {
+            registerCheck(element, aCheck);
         }
         mAllChecks.add(aCheck);
     }
@@ -340,13 +312,7 @@ public final class TreeWalker
      */
     private void registerCheck(String aToken, Check aCheck)
     {
-        ArrayList visitors = (ArrayList) mTokenToChecks.get(aToken);
-        if (visitors == null) {
-            visitors = new ArrayList();
-            mTokenToChecks.put(aToken, visitors);
-        }
-
-        visitors.add(aCheck);
+        mTokenToChecks.put(aToken, aCheck);
     }
 
     /**
@@ -359,9 +325,14 @@ public final class TreeWalker
         getMessageCollector().reset();
         notifyBegin(aAST, aContents);
 
-         // empty files are not flagged by javac, will yield aAST == null
+        // empty files are not flagged by javac, will yield aAST == null
         if (aAST != null) {
-            process(aAST);
+            if (useRecursiveAlgorithm()) {
+                processRec(aAST);
+            }
+            else {
+                processIter(aAST);
+            }
         }
 
         notifyEnd(aAST);
@@ -375,11 +346,9 @@ public final class TreeWalker
      */
     private void notifyBegin(DetailAST aRootAST, FileContents aContents)
     {
-        final Iterator it = mAllChecks.iterator();
-        while (it.hasNext()) {
-            final Check check = (Check) it.next();
-            check.setFileContents(aContents);
-            check.beginTree(aRootAST);
+        for (Check ch : mAllChecks) {
+            ch.setFileContents(aContents);
+            ch.beginTree(aRootAST);
         }
     }
 
@@ -389,18 +358,17 @@ public final class TreeWalker
      */
     private void notifyEnd(DetailAST aRootAST)
     {
-        final Iterator it = mAllChecks.iterator();
-        while (it.hasNext()) {
-            final Check check = (Check) it.next();
-            check.finishTree(aRootAST);
+        for (Check ch : mAllChecks) {
+            ch.finishTree(aRootAST);
         }
     }
 
     /**
      * Recursively processes a node calling interested checks at each node.
+     * Uses recursive algorithm.
      * @param aAST the node to start from
      */
-    private void process(DetailAST aAST)
+    private void processRec(DetailAST aAST)
     {
         if (aAST == null) {
             return;
@@ -408,18 +376,17 @@ public final class TreeWalker
 
         notifyVisit(aAST);
 
-        final DetailAST child = (DetailAST) aAST.getFirstChild();
+        final DetailAST child = aAST.getFirstChild();
         if (child != null) {
-            process(child);
+            processRec(child);
         }
 
         notifyLeave(aAST);
 
-        final DetailAST sibling = (DetailAST) aAST.getNextSibling();
+        final DetailAST sibling = aAST.getNextSibling();
         if (sibling != null) {
-            process(sibling);
+            processRec(sibling);
         }
-
     }
 
     /**
@@ -428,103 +395,97 @@ public final class TreeWalker
      */
     private void notifyVisit(DetailAST aAST)
     {
-        final ArrayList visitors =
-            (ArrayList) mTokenToChecks.get(
-                TokenTypes.getTokenName(aAST.getType()));
-        if (visitors != null) {
-            for (int i = 0; i < visitors.size(); i++) {
-                final Check check = (Check) visitors.get(i);
-                check.visitToken(aAST);
-            }
+        final Collection<Check> visitors =
+            mTokenToChecks.get(TokenTypes.getTokenName(aAST.getType()));
+        for (Check c : visitors) {
+            c.visitToken(aAST);
         }
     }
 
     /**
      * Notify interested checks that leaving a node.
-     * @param aAST the node to notify for
+     *
+     * @param aAST
+     *                the node to notify for
      */
     private void notifyLeave(DetailAST aAST)
     {
-        final ArrayList visitors =
-            (ArrayList) mTokenToChecks.get(
-                TokenTypes.getTokenName(aAST.getType()));
-        if (visitors != null) {
-            for (int i = 0; i < visitors.size(); i++) {
-                final Check check = (Check) visitors.get(i);
-                check.leaveToken(aAST);
-            }
+        final Collection<Check> visitors =
+            mTokenToChecks.get(TokenTypes.getTokenName(aAST.getType()));
+        for (Check ch : visitors) {
+            ch.leaveToken(aAST);
         }
     }
 
     /**
      * Static helper method to parses a Java source file.
-     * @param aContents contains the contents of the file
+     *
+     * @param aContents
+     *                contains the contents of the file
+     * @throws TokenStreamException
+     *                 if lexing failed
+     * @throws RecognitionException
+     *                 if parsing failed
      * @return the root of the AST
-     * @throws TokenStreamException if lexing failed
-     * @throws RecognitionException if parsing failed
      */
     public static DetailAST parse(FileContents aContents)
-        throws TokenStreamException, RecognitionException
+        throws RecognitionException, TokenStreamException
     {
-        DetailAST rootAST;
-        try {
-            // try the 1.4 grammar first, this will succeed for
-            // all code that compiles without any warnings in JDK 1.4,
-            // that should cover most cases
-            final Reader sar = new StringArrayReader(aContents.getLines());
-            final GeneratedJava14Lexer jl = new GeneratedJava14Lexer(sar);
-            jl.setFilename(aContents.getFilename());
-            jl.setCommentListener(aContents);
+        final String fullText = aContents.getText().getFullText().toString();
+        final Reader sr = new StringReader(fullText);
+        final GeneratedJavaLexer lexer = new GeneratedJavaLexer(sr);
+        lexer.setFilename(aContents.getFilename());
+        lexer.setCommentListener(aContents);
+        lexer.setTreatAssertAsKeyword(true);
+        lexer.setTreatEnumAsKeyword(true);
 
-            final GeneratedJava14Recognizer jr =
-                new SilentJava14Recognizer(jl);
-            jr.setFilename(aContents.getFilename());
-            jr.setASTNodeClass(DetailAST.class.getName());
-            jr.compilationUnit();
-            rootAST = (DetailAST) jr.getAST();
-        }
-        catch (RecognitionException re) {
-            // Parsing might have failed because the checked
-            // file contains "assert" as an identifier. Retry with a
-            // grammar that treats "assert" as an identifier
-            // and not as a keyword
+        final GeneratedJavaRecognizer parser =
+            new GeneratedJavaRecognizer(lexer);
+        parser.setFilename(aContents.getFilename());
+        parser.setASTNodeClass(DetailAST.class.getName());
+        parser.compilationUnit();
 
-            // Arghh - the pain - duplicate code!
-            final Reader sar = new StringArrayReader(aContents.getLines());
-            final GeneratedJavaLexer jl = new GeneratedJavaLexer(sar);
-            jl.setFilename(aContents.getFilename());
-            jl.setCommentListener(aContents);
-
-            final GeneratedJavaRecognizer jr = new GeneratedJavaRecognizer(jl);
-            jr.setFilename(aContents.getFilename());
-            jr.setASTNodeClass(DetailAST.class.getName());
-            jr.compilationUnit();
-            rootAST = (DetailAST) jr.getAST();
-        }
-        return rootAST;
+        return (DetailAST) parser.getAST();
     }
 
-    /** @see com.puppycrawl.tools.checkstyle.api.FileSetCheck */
-    public void process(File[] aFiles)
-    {
-        File[] javaFiles = filter(aFiles);
-
-        for (int i = 0; i < javaFiles.length; i++) {
-            process(javaFiles[i]);
-        }
-    }
-
-    /**
-     * @see com.puppycrawl.tools.checkstyle.api.FileSetCheck
-     */
+    @Override
     public void destroy()
     {
-        for (Iterator it = mAllChecks.iterator(); it.hasNext();) {
-            final Check c = (Check) it.next();
+        for (Check c : mAllChecks) {
             c.destroy();
         }
         mCache.destroy();
         super.destroy();
     }
 
+    /**
+     * @return true if we should use recursive algorithm
+     *         for tree processing, false for iterative one.
+     */
+    private boolean useRecursiveAlgorithm()
+    {
+        return mRecursive;
+    }
+
+    /**
+     * Processes a node calling interested checks at each node.
+     * Uses iterative algorithm.
+     * @param aRoot the root of tree for process
+     */
+    private void processIter(DetailAST aRoot)
+    {
+        DetailAST curNode = aRoot;
+        while (curNode != null) {
+            notifyVisit(curNode);
+            DetailAST toVisit = curNode.getFirstChild();
+            while ((curNode != null) && (toVisit == null)) {
+                notifyLeave(curNode);
+                toVisit = curNode.getNextSibling();
+                if (toVisit == null) {
+                    curNode = curNode.getParent();
+                }
+            }
+            curNode = toVisit;
+        }
+    }
 }
