@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2010  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,11 +18,10 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle.checks.indentation;
 
-import java.util.Arrays;
-
 import com.puppycrawl.tools.checkstyle.api.DetailAST;
 import com.puppycrawl.tools.checkstyle.api.TokenTypes;
 import com.puppycrawl.tools.checkstyle.api.Utils;
+import java.util.Arrays;
 
 /**
  * Abstract base class for all handlers.
@@ -34,16 +33,16 @@ public abstract class ExpressionHandler
     /**
      * The instance of <code>IndentationCheck</code> using this handler.
      */
-    private IndentationCheck mIndentCheck;
+    private final IndentationCheck mIndentCheck;
 
     /** the AST which is handled by this handler */
-    private DetailAST mMainAst;
+    private final DetailAST mMainAst;
 
     /** name used during output to user */
-    private String mTypeName;
+    private final String mTypeName;
 
     /** containing AST handler */
-    private ExpressionHandler mParent;
+    private final ExpressionHandler mParent;
 
     /** indentation amount for this handler */
     private IndentLevel mLevel;
@@ -130,15 +129,10 @@ public abstract class ExpressionHandler
     protected final void logError(DetailAST aAst, String aSubtypeName,
                                   int aActualLevel, IndentLevel aExpectedLevel)
     {
-        String typeStr = (aSubtypeName == "" ? "" : (" " + aSubtypeName));
-        Object[] args = new Object[] {
-            mTypeName + typeStr,
-            new Integer(aActualLevel),
-            aExpectedLevel,
-        };
-        mIndentCheck.indentationLog(aAst.getLineNo(),
-                                    "indentation.error",
-                                    args);
+        final String typeStr =
+            ("".equals(aSubtypeName) ? "" : (" " + aSubtypeName));
+        mIndentCheck.indentationLog(aAst.getLineNo(), "indentation.error",
+                mTypeName + typeStr, aActualLevel, aExpectedLevel);
     }
 
     /**
@@ -152,14 +146,8 @@ public abstract class ExpressionHandler
                                int aActualLevel,
                                IndentLevel aExpectedLevel)
     {
-        Object[] args = new Object[] {
-            mTypeName,
-            new Integer(aActualLevel),
-            aExpectedLevel,
-        };
-        mIndentCheck.indentationLog(aLine,
-                                    "indentation.child.error",
-                                    args);
+        mIndentCheck.indentationLog(aLine, "indentation.child.error",
+                mTypeName, aActualLevel, aExpectedLevel);
     }
 
     /**
@@ -184,8 +172,33 @@ public abstract class ExpressionHandler
      */
     static boolean areOnSameLine(DetailAST aAst1, DetailAST aAst2)
     {
-        return aAst1 != null && aAst2 != null
-            && aAst1.getLineNo() == aAst2.getLineNo();
+        return (aAst1 != null) && (aAst2 != null)
+            && (aAst1.getLineNo() == aAst2.getLineNo());
+    }
+
+    /**
+     * Searchs in given sub-tree (including given node) for the token
+     * which represents first symbol for this sub-tree in file.
+     * @param aAST a root of sub-tree in which the search shoul be performed.
+     * @return a token which occurs first in the file.
+     */
+    static DetailAST getFirstToken(DetailAST aAST)
+    {
+        DetailAST first = aAST;
+        DetailAST child = aAST.getFirstChild();
+
+        while (child != null) {
+            final DetailAST toTest = getFirstToken(child);
+            if ((toTest.getLineNo() < first.getLineNo())
+                || ((toTest.getLineNo() == first.getLineNo())
+                    && (toTest.getColumnNo() < first.getColumnNo())))
+            {
+                first = toTest;
+            }
+            child = child.getNextSibling();
+        }
+
+        return first;
     }
 
     /**
@@ -197,7 +210,7 @@ public abstract class ExpressionHandler
      */
     protected final int getLineStart(DetailAST aAst)
     {
-        String line = mIndentCheck.getLines()[aAst.getLineNo() - 1];
+        final String line = mIndentCheck.getLines()[aAst.getLineNo() - 1];
         return getLineStart(line);
     }
 
@@ -255,11 +268,12 @@ public abstract class ExpressionHandler
         }
 
         // check first line
-        int startLine = aLines.firstLine();
-        int endLine = aLines.lastLine();
-        int startCol = aLines.firstLineCol();
+        final int startLine = aLines.firstLine();
+        final int endLine = aLines.lastLine();
+        final int startCol = aLines.firstLineCol();
 
-        int realStartCol = getLineStart(mIndentCheck.getLines()[startLine - 1]);
+        final int realStartCol =
+            getLineStart(mIndentCheck.getLines()[startLine - 1]);
 
         if (realStartCol == startCol) {
             checkSingleLine(startLine, startCol, aIndentLevel,
@@ -276,14 +290,14 @@ public abstract class ExpressionHandler
         //       after complete rewriting of checkExpressionSubtree()
 
         if (aFirstLineMatches
-            || (aFirstLine > mMainAst.getLineNo() && shouldIncreaseIndent()))
+            || ((aFirstLine > mMainAst.getLineNo()) && shouldIncreaseIndent()))
         {
             aIndentLevel = new IndentLevel(aIndentLevel, getBasicOffset());
         }
 
         // check following lines
         for (int i = startLine + 1; i <= endLine; i++) {
-            Integer col = aLines.getStartColumn(new Integer(i));
+            final Integer col = aLines.getStartColumn(i);
             // startCol could be null if this line didn't have an
             // expression that was required to be checked (it could be
             // checked by a child expression)
@@ -302,8 +316,8 @@ public abstract class ExpressionHandler
      */
     private void checkSingleLine(int aLineNum, IndentLevel aIndentLevel)
     {
-        String line = mIndentCheck.getLines()[aLineNum - 1];
-        int start = getLineStart(line);
+        final String line = mIndentCheck.getLines()[aLineNum - 1];
+        final int start = getLineStart(line);
         if (aIndentLevel.gt(start)) {
             logChildError(aLineNum, start, aIndentLevel);
         }
@@ -321,14 +335,14 @@ public abstract class ExpressionHandler
     private void checkSingleLine(int aLineNum, int aColNum,
         IndentLevel aIndentLevel, boolean aMustMatch)
     {
-        String line = mIndentCheck.getLines()[aLineNum - 1];
-        int start = getLineStart(line);
+        final String line = mIndentCheck.getLines()[aLineNum - 1];
+        final int start = getLineStart(line);
         // if must match is set, it is an error if the line start is not
         // at the correct indention level; otherwise, it is an only an
         // error if this statement starts the line and it is less than
         // the correct indentation level
         if (aMustMatch ? !aIndentLevel.accept(start)
-            : aColNum == start && aIndentLevel.gt(start))
+            : (aColNum == start) && aIndentLevel.gt(start))
         {
             logChildError(aLineNum, start, aIndentLevel);
         }
@@ -344,7 +358,7 @@ public abstract class ExpressionHandler
     protected final int getLineStart(String aLine)
     {
         for (int start = 0; start < aLine.length(); start++) {
-            char c = aLine.charAt(start);
+            final char c = aLine.charAt(start);
 
             if (!Character.isWhitespace(c)) {
                 return Utils.lengthExpandedTabs(
@@ -375,9 +389,9 @@ public abstract class ExpressionHandler
                                        boolean aAllowNesting)
     {
         Arrays.sort(aTokenTypes);
-        for (DetailAST child = (DetailAST) aParent.getFirstChild();
+        for (DetailAST child = aParent.getFirstChild();
                 child != null;
-                child = (DetailAST) child.getNextSibling())
+                child = child.getNextSibling())
         {
             if (Arrays.binarySearch(aTokenTypes, child.getType()) >= 0) {
                 checkExpressionSubtree(child, aStartLevel,
@@ -401,10 +415,10 @@ public abstract class ExpressionHandler
         boolean aAllowNesting
     )
     {
-        LineSet subtreeLines = new LineSet();
-        int firstLine = getFirstLine(Integer.MAX_VALUE, aTree);
+        final LineSet subtreeLines = new LineSet();
+        final int firstLine = getFirstLine(Integer.MAX_VALUE, aTree);
         if (aFirstLineMatches && !aAllowNesting) {
-            subtreeLines.addLineAndCol(new Integer(firstLine),
+            subtreeLines.addLineAndCol(firstLine,
                 getLineStart(mIndentCheck.getLines()[firstLine - 1]));
         }
         findSubtreeLines(subtreeLines, aTree, aAllowNesting);
@@ -426,15 +440,15 @@ public abstract class ExpressionHandler
         // TODO: getLineNo should probably not return < 0, but it is for
         // the interface methods... I should ask about this
 
-        int currLine = aTree.getLineNo();
+        final int currLine = aTree.getLineNo();
         if (currLine < aStartLine) {
             aStartLine = currLine;
         }
 
         // check children
-        for (DetailAST node = (DetailAST) aTree.getFirstChild();
+        for (DetailAST node = aTree.getFirstChild();
             node != null;
-            node = (DetailAST) node.getNextSibling())
+            node = node.getNextSibling())
         {
             aStartLine = getFirstLine(aStartLine, node);
         }
@@ -446,16 +460,16 @@ public abstract class ExpressionHandler
      * Get the column number for the start of a given expression, expanding
      * tabs out into spaces in the process.
      *
-     * @param aAst   the expression to find the start of
+     * @param aAST   the expression to find the start of
      *
      * @return the column number for the start of the expression
      */
-    protected final int expandedTabsColumnNo(DetailAST aAst)
+    protected final int expandedTabsColumnNo(DetailAST aAST)
     {
-        String line =
-            mIndentCheck.getLines()[aAst.getLineNo() - 1];
+        final String line =
+            mIndentCheck.getLines()[aAST.getLineNo() - 1];
 
-        return Utils.lengthExpandedTabs(line, aAst.getColumnNo(),
+        return Utils.lengthExpandedTabs(line, aAST.getColumnNo(),
             mIndentCheck.getIndentationTabWidth());
     }
 
@@ -474,7 +488,7 @@ public abstract class ExpressionHandler
         // the interface methods... I should ask about this
 
         if (getIndentCheck().getHandlerFactory().isHandledType(aTree.getType())
-            || aTree.getLineNo() < 0)
+            || (aTree.getLineNo() < 0))
         {
             return;
         }
@@ -486,18 +500,18 @@ public abstract class ExpressionHandler
 //              return;
 //          }
 
-        Integer lineNum = new Integer(aTree.getLineNo());
-        Integer colNum = aLines.getStartColumn(lineNum);
+        final int lineNum = aTree.getLineNo();
+        final Integer colNum = aLines.getStartColumn(lineNum);
 
-        int thisLineColumn = expandedTabsColumnNo(aTree);
-        if (colNum == null || (thisLineColumn < colNum.intValue())) {
+        final int thisLineColumn = expandedTabsColumnNo(aTree);
+        if ((colNum == null) || (thisLineColumn < colNum.intValue())) {
             aLines.addLineAndCol(lineNum, thisLineColumn);
         }
 
         // check children
-        for (DetailAST node = (DetailAST) aTree.getFirstChild();
+        for (DetailAST node = aTree.getFirstChild();
             node != null;
-            node = (DetailAST) node.getNextSibling())
+            node = node.getNextSibling())
         {
             findSubtreeLines(aLines, node, aAllowNesting);
         }
@@ -508,10 +522,11 @@ public abstract class ExpressionHandler
      */
     protected final void checkModifiers()
     {
-        DetailAST modifiers = mMainAst.findFirstToken(TokenTypes.MODIFIERS);
-        for (DetailAST modifier = (DetailAST) modifiers.getFirstChild();
+        final DetailAST modifiers =
+            mMainAst.findFirstToken(TokenTypes.MODIFIERS);
+        for (DetailAST modifier = modifiers.getFirstChild();
              modifier != null;
-             modifier = (DetailAST) modifier.getNextSibling())
+             modifier = modifier.getNextSibling())
         {
             /*
             if (!areOnSameLine(modifier, prevExpr)) {
@@ -580,4 +595,49 @@ public abstract class ExpressionHandler
     {
         return getIndentCheck().getBraceAdjustement();
     }
+
+    /**
+     * Check the indentation of the right parenthesis.
+     * @param aRparen parenthesis to check
+     * @param aLparen left parenthesis associated with aRparen
+     */
+    protected final void checkRParen(DetailAST aLparen, DetailAST aRparen)
+    {
+        // no paren - no check :)
+        if (aRparen == null) {
+            return;
+        }
+
+        // the rcurly can either be at the correct indentation,
+        // or not first on the line ...
+        final int rparenLevel = expandedTabsColumnNo(aRparen);
+        if (getLevel().accept(rparenLevel) || !startsLine(aRparen)) {
+            return;
+        }
+
+        // or has <lparen level> + 1 indentation
+        final int lparenLevel = expandedTabsColumnNo(aLparen);
+        if (rparenLevel == (lparenLevel + 1)) {
+            return;
+        }
+
+        logError(aRparen, "rparen", rparenLevel);
+    }
+
+    /**
+     * Check the indentation of the left parenthesis.
+     * @param aLparen parenthesis to check
+     */
+    protected final void checkLParen(final DetailAST aLparen)
+    {
+        // the rcurly can either be at the correct indentation, or on the
+        // same line as the lcurly
+        if ((aLparen == null)
+            || getLevel().accept(expandedTabsColumnNo(aLparen))
+            || !startsLine(aLparen))
+        {
+            return;
+        }
+        logError(aLparen, "lparen", expandedTabsColumnNo(aLparen));
+    }
 }
