@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2010  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -30,5 +30,5 @@ interface IntFilter
      * @param aInt the Integer to filter.
      * @return true if the aInt is accepted.
      */
-    boolean accept(Integer aInt);
+    boolean accept(int aInt);
 }
