@@ -1,18 +1,36 @@
+////////////////////////////////////////////////////////////////////////////////
+// checkstyle: Checks Java source code for adherence to a set of rules.
+// Copyright (C) 2001-2010  Oliver Burn
+//
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
+//
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// Lesser General Public License for more details.
+//
+// You should have received a copy of the GNU Lesser General Public
+// License along with this library; if not, write to the Free Software
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle.checks.indentation;
 
-import com.puppycrawl.tools.checkstyle.BaseCheckTestCase;
-import com.puppycrawl.tools.checkstyle.DefaultConfiguration;
+import com.puppycrawl.tools.checkstyle.BaseCheckTestSupport;
 import com.puppycrawl.tools.checkstyle.Checker;
+import com.puppycrawl.tools.checkstyle.DefaultConfiguration;
+import org.junit.Test;
 
 /**
  *
  * @author  jrichard
  */
-public class IndentationCheckTest extends BaseCheckTestCase {
-
-
-    public void testInvalidLabel()
-            throws Exception
+public class IndentationCheckTest extends BaseCheckTestSupport
+{
+    @Test
+    public void testInvalidLabel() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
         final String[] expected = {
@@ -29,10 +47,8 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(checkConfig, getPath("indentation/InputInvalidLabelIndent.java"), expected);
     }
 
-
-
-    public void testValidLabel()
-            throws Exception
+    @Test
+    public void testValidLabel() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
         final String[] expected = {
@@ -40,9 +56,8 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(checkConfig, getPath("indentation/InputValidLabelIndent.java"), expected);
     }
 
-
-     public void testValidIfWithChecker()
-        throws Exception
+    @Test
+    public void testValidIfWithChecker() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
         final Checker c = createChecker(checkConfig);
@@ -52,7 +67,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-
+    @Test
     public void testValidDotWithChecker()
         throws Exception
     {
@@ -64,7 +79,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-
+    @Test
     public void testValidMethodWithChecker()
         throws Exception
     {
@@ -76,7 +91,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-
+    @Test
     public void testInvalidMethodWithChecker()
         throws Exception
     {
@@ -116,13 +131,14 @@ public class IndentationCheckTest extends BaseCheckTestCase {
             "142: method call child at indentation level 10 not at correct indentation, 12",
             "152: method call child at indentation level 6 not at correct indentation, 12",
             "158: method def throws at indentation level 6 not at correct indentation, 8",
+            "164: method def child at indentation level 4 not at correct indentation, 8",
+            "169: method def child at indentation level 4 not at correct indentation, 8",
+            "173: method def return type at indentation level 0 not at correct indentation, 4",
         };
         verify(c, fname, expected);
     }
 
-
-
-
+    @Test
     public void testInvalidSwitchWithChecker()
         throws Exception
     {
@@ -160,6 +176,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testValidSwitchWithChecker()
         throws Exception
     {
@@ -171,8 +188,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-
-
+    @Test
     public void testValidArrayInitWithChecker()
         throws Exception
     {
@@ -184,6 +200,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testInvalidArrayInitWithChecker()
         throws Exception
     {
@@ -191,11 +208,11 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         final Checker c = createChecker(checkConfig);
         final String fname = getPath("indentation/InputInvalidArrayInitIndent.java");
         final String[] expected = {
-            "17: class def child at indentation level 2 not at correct indentation, 4",
-            "18: class def child at indentation level 6 not at correct indentation, 4",
-            "20: class def child at indentation level 2 not at correct indentation, 4",
+            "17: member def type at indentation level 2 not at correct indentation, 4",
+            "18: member def type at indentation level 6 not at correct indentation, 4",
+            "20: member def type at indentation level 2 not at correct indentation, 4",
             "22: array initialization rcurly at indentation level 6 not at correct indentation, 2",
-            "24: class def child at indentation level 6 not at correct indentation, 4",
+            "24: member def type at indentation level 6 not at correct indentation, 4",
             "25: array initialization child at indentation level 8 not at correct indentation, 10",
             "26: array initialization rcurly at indentation level 4 not at correct indentation, 6",
             "29: array initialization child at indentation level 9 not at correct indentation, 8",
@@ -207,8 +224,8 @@ public class IndentationCheckTest extends BaseCheckTestCase {
             "48: array initialization child at indentation level 20 not at correct indentation, [8, 31]",
             "49: array initialization child at indentation level 4 not at correct indentation, [8, 31]",
             "54: array initialization child at indentation level 6 not at correct indentation, 8",
-            "59: class def child at indentation level 2 not at correct indentation, 4",
-            "61: class def child at indentation level 6 not at correct indentation, 4",
+            "59: member def type at indentation level 2 not at correct indentation, 4",
+            "61: member def type at indentation level 6 not at correct indentation, 4",
             "62: array initialization rcurly at indentation level 2 not at correct indentation, 6",
             "65: array initialization child at indentation level 6 not at correct indentation, 8",
             "72: array initialization child at indentation level 10 not at correct indentation, 12",
@@ -226,9 +243,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-
-
-
+    @Test
     public void testValidTryWithChecker()
         throws Exception
     {
@@ -240,6 +255,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testInvalidTryWithChecker()
         throws Exception
     {
@@ -277,8 +293,8 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-
     // TODO: needs to be finished
+    @Test
     public void testInvalidClassDefWithChecker()
         throws Exception
     {
@@ -307,19 +323,26 @@ public class IndentationCheckTest extends BaseCheckTestCase {
             "85: class def ident at indentation level 6 not at correct indentation, 4",
             "88: class def child at indentation level 2 not at correct indentation, 4",
             "88: class def ident at indentation level 2 not at correct indentation, 4",
-            "92: class def child at indentation level 6 not at correct indentation, 8",
-            "98: class def child at indentation level 10 not at correct indentation, 12",
-            "103: class def child at indentation level 6 not at correct indentation, 8",
+            "92: member def modifier at indentation level 6 not at correct indentation, 8",
+            "93: member def type at indentation level 12 not at correct indentation, 8",
+            "98: member def type at indentation level 10 not at correct indentation, 8",
+            "103: member def modifier at indentation level 6 not at correct indentation, 8",
             "108: class def rcurly at indentation level 6 not at correct indentation, 4",
             "110: class def ident at indentation level 6 not at correct indentation, 4",
             "116: class def child at indentation level 6 not at correct indentation, 8",
             "116: class def ident at indentation level 6 not at correct indentation, 8",
             "119: class def ident at indentation level 10 not at correct indentation, 8",
             "121: class def rcurly at indentation level 10 not at correct indentation, 8",
-            "124: class def child at indentation level 10 not at correct indentation, 12",
+            "124: member def type at indentation level 10 not at correct indentation, 12",
             "129: method def child at indentation level 10 not at correct indentation, 8",
-            "138: object def lcurly at indentation level 6 not at correct indentation, 8",
-            "142: object def rcurly at indentation level 6 not at correct indentation, 8",
+            "130: object def lcurly at indentation level 8 not at correct indentation, [10, 14]",
+            "134: object def rcurly at indentation level 8 not at correct indentation, [10, 14]",
+            "138: object def lcurly at indentation level 6 not at correct indentation, [8, 12]",
+
+            "139: method def modifier at indentation level 12 not at correct indentation, 10",
+            "141: method def rcurly at indentation level 12 not at correct indentation, 10",
+
+            "142: object def rcurly at indentation level 6 not at correct indentation, [8, 12]",
             "147: method def modifier at indentation level 10 not at correct indentation, 12",
             "149: method def rcurly at indentation level 10 not at correct indentation, 12",
             // TODO: extends and implements need to be added
@@ -327,7 +350,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-
+    @Test
     public void testInvalidBlockWithChecker()
         throws Exception
     {
@@ -386,6 +409,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testInvalidIfWithChecker()
         throws Exception
     {
@@ -475,6 +499,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testInvalidWhileWithChecker()
         throws Exception
     {
@@ -509,10 +534,12 @@ public class IndentationCheckTest extends BaseCheckTestCase {
             "74: while rparen at indentation level 5 not at correct indentation, 8",
             "81: while rparen at indentation level 10 not at correct indentation, 8",
             "88: while rparen at indentation level 10 not at correct indentation, 8",
+            "95: while child at indentation level 8 not at correct indentation, 12",
         };
         verify(c, fname, expected);
     }
 
+    @Test
     public void testInvalidForWithChecker()
         throws Exception
     {
@@ -544,6 +571,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testValidForWithChecker()
         throws Exception
     {
@@ -555,6 +583,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testValidDoWhileWithChecker()
         throws Exception
     {
@@ -566,6 +595,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testValidBlockWithChecker()
         throws Exception
     {
@@ -578,6 +608,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
     }
 
 
+    @Test
     public void testValidWhileWithChecker()
         throws Exception
     {
@@ -589,6 +620,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testValidClassDefWithChecker()
         throws Exception
     {
@@ -600,6 +632,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testValidInterfaceDefWithChecker()
         throws Exception
     {
@@ -611,6 +644,7 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
+    @Test
     public void testValidCommaWithChecker()
         throws Exception
     {
@@ -623,51 +657,52 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(c, fname, expected);
     }
 
-    public void testTabs()
-            throws Exception
+    @Test
+    public void testTabs() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
-        checkConfig.addAttribute("basicOffset", new Integer(4).toString());
-        checkConfig.addAttribute("tabWidth", new Integer(4).toString());
+        checkConfig.addAttribute("basicOffset", Integer.valueOf(4).toString());
+        checkConfig.addAttribute("tabWidth", Integer.valueOf(4).toString());
         final String[] expected = {
             "25: ctor def child at indentation level 9 not at correct indentation, 8",
         };
         verify(checkConfig, getPath("indentation/InputUseTabs.java"), expected);
     }
 
-    public void testIndentationLevel()
-            throws Exception
+    @Test
+    public void testIndentationLevel() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
-        checkConfig.addAttribute("basicOffset", new Integer(2).toString());
+        checkConfig.addAttribute("basicOffset", Integer.valueOf(2).toString());
         final String[] expected = {
             "25: ctor def child at indentation level 5 not at correct indentation, 4",
         };
         verify(checkConfig, getPath("indentation/InputUseTwoSpaces.java"), expected);
     }
 
-    public void testCaseLevel()
-            throws Exception
+    @Test
+    public void testCaseLevel() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
-        checkConfig.addAttribute("caseIndent", new Integer(0).toString());
+        checkConfig.addAttribute("caseIndent", Integer.valueOf(0).toString());
         final String[] expected = {
             "23: case child at indentation level 10 not at correct indentation, 8",
         };
         verify(checkConfig, getPath("indentation/InputCaseLevel.java"), expected);
     }
 
-    public void testBraceAdjustment()
-            throws Exception
+    @Test
+    public void testBraceAdjustment() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
-        checkConfig.addAttribute("braceAdjustment", new Integer(2).toString());
+        checkConfig.addAttribute("braceAdjustment", Integer.valueOf(2).toString());
         final String[] expected = {
             "24: if rcurly at indentation level 8 not at correct indentation, 10",
         };
         verify(checkConfig, getPath("indentation/InputBraceAdjustment.java"), expected);
     }
 
+    @Test
     public void testInvalidAssignWithChecker() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
@@ -681,10 +716,19 @@ public class IndentationCheckTest extends BaseCheckTestCase {
         verify(checkConfig, getPath("indentation/InputInvalidAssignIndent.java"), expected);
     }
 
+    @Test
     public void testValidAssignWithChecker() throws Exception
     {
         final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
         final String[] expected = {};
         verify(checkConfig, getPath("indentation/InputValidAssignIndent.java"), expected);
     }
+
+    @Test
+    public void test15Extensions() throws Exception
+    {
+        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
+        final String[] expected = {};
+        verify(checkConfig, getPath("Input15Extensions.java"), expected);
+    }
 }
